@@ -128,7 +128,7 @@ parts."
   :type 'boolean
   :group 'oai)
 
-(defcustom oai-block-fill-function #'oai-block--fill-region
+(defcustom oai-block-fill-function #'oai-block-fill-insert
   "If non-nil this function will be called after insertion of text.
 Current buffer is buffer with ai block with position of pointer right
 after insertion of text.
@@ -213,6 +213,8 @@ In `oai-block-roles-prefixes'.")
 
 (defconst oai-block--ai-block-begin-re "^#\\+begin_ai.*$")
 (defconst oai-block--ai-block-end-re "^#\\+end_ai.*$")
+(defconst oai-block--ai-block-begin-end-re "^#\\+\\(begin\\|end\\)_ai.*$")
+
 
 (defvar oai-block--markdown-begin-re "^\\s-*```\\([^\s\t\n[{]+\\)[\s\t]?$")
 (defvar oai-block--markdown-end-re "^[\s\t]**```\\s-*$")
@@ -220,6 +222,8 @@ In `oai-block-roles-prefixes'.")
 (defvar oai-block--chat-prefixes-re "^\\s-*\\[\\([^\]]+\\)\\(:\\]\\|\\]:\\)\\s-*"
   "Prefix should be at the begining of the line with spaces or without.
 Or roles regex.")
+(defvar oai-block--markdown-header-re "^\\(#+\\)\\s-+\\([0-9a-zA-Z][).]\\)?\\s-*\\(.*\\)$"
+  "Used for highlighting and for jumping.")
 
 
 (defface oai-block--me-ai-chat-prefixes-font-face
@@ -237,16 +241,24 @@ You can customize this font with `set-face-attribute'."
   `(cl-letf (((symbol-function #'org-element--cache-active-p) (lambda (&rest _) nil)))
      ,@body))
 
-(defun oai-block-p ()
-  "Are we inside ai block?
+(defun oai-block-p (&optional element)
+  "Check if point at ai block or ELEMENT is ai block if provided.
+Optional argument ELEMENT is returned by `org-element-at-point', when
+ non-nil, checked if it is ai block, if not nil is retuned.
+If ELEMENT is not provider, current position in buffer used to get ai
+ block.
 Like `org-in-src-block-p'.  Return element."
-  (oai-block--org-element-with-disabled-cache ;; with cache enabled we get weird Cached element is incorrect warnings
-    (cl-loop with context = (org-element-context)
-             while (and context
-                        (not (equal 'special-block (org-element-type context)))
-                        (not (string-equal "ai" (org-element-property :type context))))
-             do (setq context (org-element-property :parent context))
-             finally return context)))
+  (if (and element
+           (string-equal "ai" (org-element-property :type element)))
+      element
+    ;; else
+    (oai-block--org-element-with-disabled-cache ;; with cache enabled we get weird Cached element is incorrect warnings
+      (cl-loop with context = (org-element-at-point)
+               while (and context
+                          (not (equal 'special-block (org-element-type context)))
+                          (not (string-equal "ai" (org-element-property :type context))))
+               do (setq context (org-element-property :parent context))
+               finally return context))))
 
 ;; -=-= info fn: get-info, get-request-type, get-sys
 (defun oai-block-get-info (&optional element no-eval)
@@ -835,6 +847,7 @@ block, may be retrieved with :contents-begin and :contents-end
 properties of ai block Org element.
 MAX-TOKEN-RECOMMENDATION is string to add to first system message.
 Return vector with plist with :role and :content."
+  (oai--debug "oai-block--collect-chat-messages" content-start content-end default-system-prompt persistant-sys-prompts max-token-recommendation separator)
   (let* ((separator (or separator "\n"))
          ;; 1) Positions: for prefixes [ME:], [AI:] in current buffer
          (positions (oai-block--get-chat-messages-positions content-start content-end oai-block--chat-prefixes-re))
@@ -1110,29 +1123,7 @@ on the current line.
   (or (oai-block--in-markdown-quotes-p pos "`")
       (oai-block--in-markdown-quotes-p pos "```")))
 
-;; -=-= interactive: next-ai-block
-(defvar oai-block-ai-block-regexp
-  (concat
-   "^#\\+begin_ai[^\n]*\n"
-   ;; (1) body
-   "\\(\\(?:.\\|\n\\)*?\\)??[ \t\n]*#\\+end_ai")
-  "Regexp used to ai blocks or its body.
-Modified `org-babel-src-block-regexp' for `oai-block-next-ai-block'.")
-
-(defun oai-block-next-ai-block (&optional arg)
-  "Jump to the next ai block.
-Like `org-babel-next-src-block'.
-With optional prefix argument ARG, jump forward ARG many source blocks."
-  (interactive "p")
-  (org-next-block arg nil oai-block-ai-block-regexp))
-
-(defun oai-block-previous-ai-block (&optional arg)
-  "Jump to the previous ai block.
-With optional prefix argument ARG, jump backward ARG many source blocks."
-  (interactive "p")
-  (org-previous-block arg oai-block-ai-block-regexp))
-
-;; -=-= Interactive
+;; -=-= Interactive: mark-at-point
 
 (defun oai-block-mark-last-region ()
   "Mark the last prompt in an oai block."
@@ -1143,8 +1134,9 @@ With optional prefix argument ARG, jump backward ARG many source blocks."
         (goto-char last-region-end)
         (push-mark last-region-start t t)))
 
-(defun oai-block-mark-chat-message-at-point ()
-  "Mark the prompt at point: [ME:], [AI:]."
+(defun oai-block-mark-chat-message-at-point (&optional not-mark)
+  "Mark the prompt at point: [ME:], [AI:].
+If optional argument NOT-MARK is non-nil dont activate transient-mode."
   (interactive)
   (oai--debug "oai-block-mark-chat-message-at-point1 %s" (point))
   (when-let* ((regions (oai-block--chat-role-regions))
@@ -1160,59 +1152,121 @@ With optional prefix argument ARG, jump backward ARG many source blocks."
       ;; (goto-char end)
       ;; (while (bolp)
       ;;       (backward-char))
-      (goto-char start)
-      (unless (region-active-p) (push-mark end t t))
+      (unless (or not-mark (region-active-p))
+        (goto-char start)
+        (push-mark end t t))
       (cons start end))))
 
+;; -=-= Interactive: jump forward/backward
+
+(defun oai-block--find-next-prev-region (direction current-point regions)
+  "Helper to find the next or previous region boundary.
+DIRECTION is the movement direction: negative for previous, positive for
+ next.
+CURRENT-POINT is the current cursor position. REGIONS is the list of all
+ region boundaries.
+Returns the position of the region boundary or nil if not found."
+  (if (> direction 0)
+      (seq-find (lambda (r) (> r current-point)) regions) ;; Next region
+    (seq-find (lambda (r) (< r current-point)) (reverse regions)))) ;; Previous region
+
 (defun oai-block-next-message (&optional arg)
-  "Move to next message or to the end of current in current ai block.
-A negative argument ARG = -N means move backward."
+  "Navigate between AI block messages based on ARG.
+ARG may be nil, forward if positive or backward if negative between
+ roles in ai block."
   (interactive "^p")
-  ;;   TODO:
-  ;; With argument ARG, do it ARG times;
-  ;; a negative argument ARG = -N means move backward N paragraphs.
-  (when-let* ((regions (oai-block--chat-role-regions))
-              (start (cl-find-if (lambda (x) (>= (point) x)) (reverse regions)))
-              (end (cl-find-if (lambda (x) (< (point) x)) regions)))
-    ;; (oai--debug "oai-forward-section1 %s %s" start end)
-    (or arg (setq arg 1))
-    (if (> arg 0)
-        (progn (unless (region-active-p) (push-mark nil t)) ; save position
-               (goto-char end))
-      ;; else - backward
-      (let ((prev (cl-find-if (lambda (x) (>= (1- start) x)) (reverse regions))))
-        ;; (oai--debug "oai-forward-section2 %s %s %s" (>= (point) start) prev start)
-        (when  prev ;; (>= (point) start))
-          (if (and (> (point) start) ; if at the middle of first section
-                   (not prev))
-              (progn (unless (region-active-p) (push-mark nil t)) ; save position
-                     (goto-char start))
-            ;; else
-            (unless (region-active-p) (push-mark nil t)) ; save position
-            (goto-char (cl-find-if (lambda (x) (>= (1- start) x)) (reverse regions)))))))))
+  (or arg (setq arg 1))
+  (when (and arg (< arg 0))
+    (forward-line -1))
+  (let* ((regions (oai-block--chat-role-regions))
+         (current-point (point))
+         (arg (or arg 1)) ;; Default to forward movement
+         (target-region (oai-block--find-next-prev-region arg current-point regions)))
+    ;; Save cursor position if no region is active
+    (unless (region-active-p) (push-mark nil t))
+    ;; Jump to the target region if it exists
+    (when target-region
+      (goto-char target-region))))
 
 (defun oai-block-previous-message (&optional arg)
-  "Move to previous message or to the begining of current in current ai block.
+  "Call `org-previous-visible-heading' or jump to previous ai message.
+Work if cursor in ai block.
+If at the first message, jump to the begining of current ai block.
 A positive ARG = N means move backward N sections."
   (interactive "^p")
   (oai-block-next-message (- (or arg 1))))
 
+(defun oai-block-next-item (&optional arg)
+  "Jump forward/backward by items, item type detected by cursor position.
+Optional ARG may be positive or negative to indicate direction and
+ steps."
+  (interactive "^p")
+  (or arg (setq arg 1))
+  (let* ((reg (oai-block--contents-area))
+         (beg (car reg))
+         (end (cdr reg)))
+  (cond
+   ;; begin/end of ai block
+   ((save-excursion
+      (move-beginning-of-line 1)
+      (looking-at oai-block--ai-block-begin-end-re))
+    (oai--debug "oai-block-next-item 1")
+    (end-of-line)
+    (while (/= arg 0)
+      (if (> arg 0)
+          (progn
+            (end-of-line)
+            (re-search-forward oai-block--ai-block-begin-end-re nil t)
+            (setq arg (1- arg)))
+        (beginning-of-line)
+        (re-search-backward oai-block--ai-block-begin-end-re nil t)
+        (setq arg (1+ arg)))
+      (beginning-of-line)))
+   ;; markdown-headers
+   ((save-excursion
+      (move-beginning-of-line 1)
+      (looking-at oai-block--markdown-header-re))
+    (oai--debug "oai-block-next-item 2")
+    (end-of-line)
+    (while (/= arg 0)
+      (if (> arg 0)
+          (progn
+            (end-of-line)
+            (re-search-forward oai-block--markdown-header-re end t)
+            (setq arg (1- arg)))
+        (beginning-of-line)
+        (re-search-backward oai-block--markdown-header-re beg t)
+        (setq arg (1+ arg)))
+      (beginning-of-line)))
+   ;; markdown-block beg/end
+   ((save-excursion
+      (move-beginning-of-line 1)
+      (looking-at oai-block--markdown-beg-end-re))
+    (oai--debug "oai-block-next-item 3")
+    (while (/= arg 0)
+      (if (> arg 0)
+          (progn
+            (end-of-line)
+            (re-search-forward oai-block--markdown-beg-end-re end t)
+            (setq arg (1- arg)))
+        (beginning-of-line)
+        (re-search-backward oai-block--markdown-beg-end-re beg t)
+        (setq arg (1+ arg)))
+      (beginning-of-line)))
+   ;; message
+   (t
+    (oai--debug "oai-block-next-item 4")
+    (if (> arg 0)
+        (oai-block-next-message arg)
+      (oai-block-next-message arg))))))
 
- ;; (defun oai-block-next-element (&optional arg)
+(defun oai-block-previous-item (&optional arg)
+  "Jump backward by items, item type detected by cursor position.
+ARG should be positive number or nil."
+  (interactive "^p")
+  (oai-block-next-item (- (or arg 1))))
 
-;; (defun oai-block-kill-message-at-point (&optional arg)
-;;   "Kill the prompt at point.
-;; The numeric ARG can be used for killing the last n."
-;;   (interactive "P")
-;;   (if arg
-;;       (when-let* ((region (oai-block--area))
-;;                   (start (car region))
-;;                   (end (cdr region)))
-;;         (kill-region end start))
-;;     (when-let* ((region (oai-block-mark-chat-message-at-point))
-;;                 (start (car region))
-;;                 (end (cdr region)))
-;;       (kill-region end start))))
+;; -=-= Interactive: mark-at-point
 
 (defun oai-block-mark-at-point-by-steps ()
   "Progressively mark a larger region in AI block at point.
@@ -1327,17 +1381,17 @@ If universal argument ARG is non-nil, mark content of ai block."
     ;; else
     (if (not arg)
         (cond
-         ; at header
+         ;; at header
          ((save-excursion
             (move-beginning-of-line 1)
-            (looking-at oai-block--ai-block-begin-re))
+            (looking-at oai-block--ai-block-begin-end-re))
           (let* ((reg (save-excursion (oai-block--area)))
                  (beg (car reg))
                  (end (cdr reg)))
             (goto-char beg)
             (set-mark end)
             (message "Block whole")))
-         ; at message
+         ;; at message
          ((save-excursion
             (move-beginning-of-line 1)
             (looking-at oai-block--chat-prefixes-re))
@@ -1352,62 +1406,8 @@ If universal argument ARG is non-nil, mark content of ai block."
         (goto-char beg)
         (set-mark end)))))
 
-  ;; (if (and (not arg)
-  ;;          (region-active-p))
-
-  ;;     ;; else
-  ;;     (if arg
-  ;;         (progn (deactivate-mark)
-  ;;                (oai-block-mark-chat-message-at-point)
-  ;;                (message "Chat message"))
-        ;; else
-
-;; (defun oai-block-next-element (&optional arg)
-;;   "Select next element of ai block or just jump.
-;; If optional argument ARG is non-nil, use default behavior to just jump.
-;; Used with conjunction of `oai-block-mark-at-point' function.
-;; If region is active and no ARG we select next element by selecting it as
-;; current."
-;;   (interactive "P")
-;;   (steps (list 'org-forward-element ;0
-;;                '
-;;                (if (region-active-p)
-;;       (let (
-;;                          )
-;;             (mark-num (save-mark-and-excursion
-;;                         (oai-block-mark-at-point)))) ;
-;;         ;; (deactivate-mark)
-;;         (print mark-num)
-;;          ))
-;;         ;; (cond
-;;         ;;  (eq mark-num
-;;         ;; oai-block-next-message
-;;     ;; else
-;;     (cond
-;;      ((org-in-regexp oai-block--chat-prefixes-re)
-;;       (re-search-forward oai-block--chat-prefixes-re nil t)) ; todo limit current block
-;;     ))
-
-
-
-;; not used for now
-(defun oai-find-named-block (name)
-  "Find block by NAME from begining of current buffer.
-Return the location of the block identified by source
-NAME, or nil if no such block exists.
-    Like `org-babel-find-named-block'."
-  (save-excursion
-    (goto-char (point-min))
-    (let ((regexp (concat org-babel-src-name-regexp
-	                  (concat (if name (regexp-quote name) "\\(?9:.*?\\)") "[ \t]*" )
-	                  "\\(?:\n[ \t]*#\\+\\S-+:.*\\)*?"
-	                  "\n")))
-      (or (and (looking-at regexp)
-	       (progn (goto-char (match-beginning 0))
-                      (line-beginning-position)))
-          (ignore-errors (org-next-block 1 nil regexp))))))
-
 
+;; -=-= fn: set-block-parameter
 
 (defun oai-block-set-block-parameter (parameter default-value)
   "Jump to header of ai block and set PARAMETER.
@@ -1433,6 +1433,24 @@ Used for `oai-set-max-tokens-org' function."
     ;; else
     (message "Not oai block here.")))
 
+;; -=-= fn: find named block (Not used)
+(defun oai-find-named-block (name)
+  "Find block by NAME from begining of current buffer.
+Return the location of the block identified by source
+NAME, or nil if no such block exists.
+    Like `org-babel-find-named-block'."
+  (save-excursion
+    (goto-char (point-min))
+    (let ((regexp (concat org-babel-src-name-regexp
+	                  (concat (if name (regexp-quote name) "\\(?9:.*?\\)") "[ \t]*" )
+	                  "\\(?:\n[ \t]*#\\+\\S-+:.*\\)*?"
+	                  "\n")))
+      (or (and (looking-at regexp)
+	       (progn (goto-char (match-beginning 0))
+                      (line-beginning-position)))
+          (ignore-errors (org-next-block 1 nil regexp))))))
+
+
 ;; -=-= Markers
 
 (defun oai-block-element-by-marker (marker)
@@ -1683,7 +1701,7 @@ Executed in `font-lock-defaults' chain."
 Argument START END are block begin and end, used as limits here."
   (goto-char start)
 
-  (while (re-search-forward "^\\(#+\\)\\s-+\\([0-9a-zA-Z][).]\\)?\\s-*\\(.*\\)$" end t)
+  (while (re-search-forward oai-block--markdown-header-re end t)
     ;; (print (point))
 
     (let ((b1 (match-beginning 1))
@@ -1985,78 +2003,118 @@ fill-region-as-paragraph."
     (unless (bolp)
       (forward-line))))
 
+
+(defun oai-block-fill-region (beg end &optional justify)
+  "Ignore code blocks that start with '```sometext' and end with '```'.
+BEG END and JUSTIFY have same as in `fill-region-as-paragraph'.
+TODO: use `forward-paragraph' instead of `forward-line'."
+  (let ((modified-flag (buffer-chars-modified-tick))
+        ;; markdown block range
+        block-start block-end)
+    ;; Content exist?
+    (when (or (not beg)
+              (not end))
+      (error "Empty block"))
+    ;; Ignore code blocks that start with "```sometext" and end with "```"
+    (save-excursion
+      (while (< beg end)
+        (goto-char beg)
+        ;; - search forward for markdkown begining from ai block begining
+        (if (re-search-forward "^[ \t\f]*```\\w" end t) ; ex. "    ```elisp"
+            (progn
+              ;; (setq block-start (copy-marker (line-beginning-position))) ; save markdown block begining
+              (setq block-start (line-beginning-position)) ; save markdown block begining
+              ;; - search forward from current for markdown ending
+              (if (re-search-forward "^[ \t\f]*```[ \t\f]*$" end t) ; ex. "    ```      "
+                  (progn
+                    (setq block-end (copy-marker (line-beginning-position)))
+                    ;; from begining of ai block to begin to markdown block
+                    ;; FILL
+                    (oai-block--apply-to-region-lines #'oai-block-fill-region-as-paragraph beg block-start justify)
+                    ;; go to the end of markdown block
+                    (goto-char (marker-position block-end))
+                    (set-marker block-end nil)
+                    (forward-line 1)
+                    (setq beg (point)))
+                ;; else - not found end of block
+                ;; FILL
+                (oai-block--apply-to-region-lines #'oai-block-fill-region-as-paragraph beg block-start justify)
+                ;; (set-marker block-start nil)
+                (setq beg end)))
+          ;; else - no markdown block begining - apply to whole block
+          ;; FILL
+          (oai-block--apply-to-region-lines #'oai-block-fill-region-as-paragraph beg end justify)
+          (setq beg end)))
+      (/= (buffer-chars-modified-tick) modified-flag))))
+
 (defun oai-block-fill-paragraph (&optional justify region)
   "Fill every line as paragraph in the current AI block.
-Ignore code blocks that start with '```sometext' and end with '```'.
+
 Optional argument JUSTIFY is parameter of `fill-paragraph'.
-Optional argument REGION todo.
-TODO: use `forward-paragraph' instead of `forward-line'.
-Was causing freezing."
+
+The REGION argument is non-nil if called interactively; in that
+case, if Transient Mark mode is enabled and the mark is active,
+fill each of the elements in the active region, instead of just
+filling the current element.
+Return t if point at ai block, nil otherwise."
   (interactive (progn
                  (barf-if-buffer-read-only)
-                 (list (when current-prefix-arg 'full) t)))
-  (ignore region)
+                 (list (when current-prefix-arg 'full)
+                       (and (region-active-p)
+                            (not (= (region-beginning) (region-end)))))))
   ;; inspired by `org-fill-element'
-  (oai--debug "oai-block-fill-paragraph %s %s" (point) (current-buffer))
+  (oai--debug "oai-block-fill-paragraph1 %s %s %s %s" justify region (point) (current-buffer) (region-active-p))
 
   (with-syntax-table org-mode-transpose-word-syntax-table
-    (let ((element (oai-block-p))) ; TODO: replace with last replay only
-      (when element
-        ;; Determine the boundaries of the content
-        (let ( ;; ai block range
-              (beg (org-element-property :contents-begin element)) ; first line of content
-              (end (org-element-property :contents-end element)) ; begining of #+end_ai
-              ;; markdown block range
-              block-start block-end)
-          ;; Content exist?
-          (when (or (not beg)
-                  (not end))
-              (error "Empty block"))
-          ;; Ignore code blocks that start with "```sometext" and end with "```"
-          (save-excursion
-            (while (< beg end)
-              (goto-char beg)
-              ;; - search forward for markdkown begining from ai block begining
-              (if (re-search-forward "^[ \t\f]*```\\w" end t) ; ex. "    ```elisp"
-                  (progn
-                    ;; (setq block-start (copy-marker (line-beginning-position))) ; save markdown block begining
-                    (setq block-start (line-beginning-position)) ; save markdown block begining
-                    ;; - search forward from current for markdown ending
-                    (if (re-search-forward "^[ \t\f]*```[ \t\f]*$" end t) ; ex. "    ```      "
-                        (progn
-                          (setq block-end (copy-marker (line-beginning-position)))
-                          ;; from begining of ai block to begin to markdown block
-                          ;; FILL
-                          (oai-block--apply-to-region-lines #'oai-block-fill-region-as-paragraph beg block-start justify)
-                          ;; go to the end of markdown block
-                          (goto-char (marker-position block-end))
-                          (set-marker block-end nil)
-                          (forward-line 1)
-                          (setq beg (point)))
-                      ;; else - not found end of block
-                      ;; FILL
-                      (oai-block--apply-to-region-lines #'oai-block-fill-region-as-paragraph beg block-start justify)
-                      ;; (set-marker block-start nil)
-                      (setq beg end)))
-                ;; else - no markdown block begining - apply to whole block
-                ;; FILL
-                (oai-block--apply-to-region-lines #'oai-block-fill-region-as-paragraph beg end justify)
-                (setq beg end)))
-            t))))))
+    ;; Determine the boundaries of the content
+    (when-let* ((element (oai-block-p))
+                (reg (oai-block--contents-area element))
+                (beg (car reg))
+                (end (cdr reg)))
+
+      (unless (= beg end)
+        (oai--debug "oai-block-fill-paragraph11 %s %s %s %s" region (region-active-p))
+        (when-let* ((reg
+                     (cond
+                      ;; region
+                      (region
+                       (oai--debug "oai-block-fill-paragraph12 %s %s %s %s" region (region-active-p))
+                       (let ((rbeg (region-beginning))
+                             (rend (region-end)))
+                         (oai--debug "oai-block-fill-paragraph13 %s %s %s %s" rbeg rend beg end)
+                         (cons rbeg rend)))
+                      ;; (oai-block-fill-region rbeg rend)))
+                      ;; at header
+                      ((save-excursion
+                         (move-beginning-of-line 1)
+                         (looking-at oai-block--ai-block-begin-end-re))
+                       (when (called-interactively-p 'any)
+                         (message "Block content"))
+                       (cons beg end))
+                      ;; at message
+                      ((save-excursion
+                         (move-beginning-of-line 1)
+                         (looking-at oai-block--chat-prefixes-re))
+                       (when (called-interactively-p 'any)
+                         (message "Chat message"))
+                       (oai-block-mark-chat-message-at-point t))))
+                    (beg (car reg))
+                    (end (cdr reg)))
+          (oai--debug "oai-block-fill-paragraph2 %s" beg end)
+          (oai-block-fill-region beg end)) ; return t
+        ))))
 
 
-(defun oai-block--fill-region (&optional pos stream)
+(defun oai-block-fill-insert (&optional pos stream)
   "Fill ai block for not streaming and for streaming.
-If STREAM is non-nil this function  called after insertion of a chink of
-text, otherwise after insertion of full response.
 Ignore markdown blocks, quoted text and Org tables.
-Optional argument POS position before insertion not used, TODO: set to
-to begin of paragraph."
-
+If STREAM is non-nil this function called after insertion of a chink of
+ text, use current cursor position and fill current line.
+If STREAM is nil, then region from pos to current position is filled.
+POS is position before insertion."
   (interactive)
-  (ignore pos)
   ;; (setq _pos _pos) ; for melpazoid
-  (oai--debug "oai-block--fill-region %s %s" stream (point))
+  (oai--debug "oai-block--fill-insert %s %s" stream (point) (region-active-p))
   (save-excursion
     (if stream
         ;; if at current line ``` or we are at begining of markdown block in ai block.
@@ -2081,8 +2139,8 @@ to begin of paragraph."
             (goto-char p)
             (fill-region-as-paragraph (line-beginning-position) (line-end-position))))
       ;; else not stream, single response. We add hack to skip markdown blocks.
-      (oai-block-fill-paragraph) ; fill per line. wrapped in save-excursion inside.
-      )))
+      (let ((p (or pos (line-beginning-position))))
+        (oai-block-fill-region p (point))))))
 
 ;; (defun my/org-fill-element-advice (orig-fun &optional justify)
 ;;   "Advice around `org-fill-element`.
@@ -2154,18 +2212,24 @@ Info about arguments NO-EVAL DATUM in `org-babel-get-src-block-info'."
       info)))
 
 
-;; (defun oai-block--org-babel-where-is-src-block-head (&optional src-block)
-;;   "`org-babel-where-is-src-block-head'."
-;;   )
+(defun oai-block--org-babel-where-is-src-block-head-advice (orig-fun &rest args)
+  "Advice for `org-babel-tangle' function and some.
+ORIG-FUN is `org-babel-where-is-src-block-head' and its ARGS."
+  (if-let ((element (or (and args (oai-block-p (car args)))
+                      (oai-block-p))))
+      (org-element-property :begin element)
+    ;; else
+  (apply orig-fun args)))
 
 
 (defun oai-block--org-babel-get-src-block-info-advice (orig-fun &rest args)
   "Advice for `org-babel-tangle' function and some.
 ORIG-FUN is `oai-block--org-babel-get-src-block-info' and its ARGS."
-  (if (not (oai-block-p))
-      (apply orig-fun args)
-    ;; else
-    (apply orig-fun args)))
+  (seq-let (no-eval datum) args
+    (if-let ((datum (or (oai-block-p datum) (oai-block-p))))
+      (oai-block--org-babel-get-src-block-info no-eval datum)
+      ;; else
+      (apply orig-fun args))))
 
 
 ;;;; provide
